<?php
/**
 * Genie storage implementation for PHP's PEAR database abstraction layer.
 *
 * Required values for $params:<pre>
 *      'phptype'       The database type (e.g. 'pgsql', 'mysql', etc.).
 *      'hostspec'      The hostname of the database server.
 *      'protocol'      The communication protocol ('tcp', 'unix', etc.).
 *      'username'      The username with which to connect to the database.
 *      'password'      The password associated with 'username'.
 *      'database'      The name of the database.
 *      'table'         The name of the items table in 'database'.</pre>
 *
 * Required by some database implementations:
 *      'options'       Additional options to pass to the database.
 *      'tty'           The TTY on which to connect to the database.
 *      'port'          The port on which to connect to the database.
 *
 * The table structure can be created by the scripts/drivers/genie_items.sql
 * script.
 *
 * $Horde: genie/lib/Driver/sql.php,v 1.14 2004/10/08 15:45:22 chuck Exp $
 *
 * @author  Jon Parise <jon@horde.org>
 * @version $Revision: 1.14 $
 * @since   Genie 0.1
 * @package Genie
 */
class Genie_Driver_sql extends Genie_Driver {

    /**
     * Handle for the current database connection.
     * @var object DB $db
     */
    var $_db;

    /**
     * Are we connected to the SQL server?
     * @var boolean $_connected
     */
    var $_connected = false;

    /**
     * Construct a new SQL storage object.
     *
     * @param string $wishlist  The wishlist to access.
     * @param array  $params    A hash containing connection parameters.
     */
    function Genie_Driver_sql($wishlist, $params = array())
    {
        $this->_wishlist = $wishlist;
        $this->_params = $params;
    }

    /**
     * Retrieve one item from the database.
     *
     * @param integer $itemId  The id of the item to retrieve.
     *
     * @return array  The array of item attributes.
     */
    function get($itemId)
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        /* Build the SQL query. */
        $query = sprintf('SELECT * FROM %s WHERE wish_owner = %s and wish_id = %s',
                         $this->_params['table'], $this->_db->quote($this->_wishlist), (int)$itemId);

        /* Log the query at a DEBUG log level. */
        Horde::logMessage(sprintf('Genie_Driver_sql::get(): %s', $query),
                          __FILE__, __LINE__, PEAR_LOG_DEBUG);

        /* Execute the query. */
        $result = $this->_db->query($query);

        if (is_a($result, 'PEAR_Error')) {
            return $result;
        }

        $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
        if (is_a($row, 'PEAR_Error')) {
            return $row;
        }

        /* Create a new item based on this row's values. */
        $item = array();
        $item['item_id'] = $row['wish_id'];
        $item['wishlist_id'] = $this->_wishlist;
        $item['name'] = $this->_convertFromDriver($row['wish_name']);
        $item['desc'] = $this->_convertFromDriver($row['wish_desc']);
        $item['quantity'] = $row['wish_quantity'];
        $item['price'] = $this->_convertFromDriver($row['wish_price']);
        $item['prodid'] = $this->_convertFromDriver($row['wish_prodid']);
        $item['prodid_type'] = $this->_convertFromDriver($row['wish_prodid_type']);
        $item['vendors'] = $this->_convertFromDriver($row['wish_vendors']);
        $item['category'] = $this->_convertFromDriver($row['wish_category']);
        $item['purchased'] = $row['wish_purchased'];
        $item['purchased_on'] = $row['wish_purchased_on'];
        $item['purchased_by'] = $this->_convertFromDriver($row['wish_purchased_by']);

        return $item;
    }

    /**
     * Add a item to the backend storage.
     *
     * @return integer  The numeric ID of the new item.
     */
    function add($name, $desc, $quantity, $price, $prodid,
                 $prodid_type, $vendors, $category, $purchased,
                 $purchased_on = null, $purchased_by = '')
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        $itemId = $this->_db->nextId($this->_params['table']);
        if (is_a($itemId, 'PEAR_Error')) {
            return $itemId;
        }

        $query = sprintf(
            'INSERT INTO %s (wish_owner, wish_id, wish_name, ' .
            'wish_desc, wish_quantity, wish_price, ' .
            'wish_prodid, wish_prodid_type, wish_vendors, ' .
            'wish_category, wish_purchased, ' .
            'wish_purchased_on, wish_purchased_by) ' .
            'VALUES (%s, %d, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s, %s)',
            $this->_params['table'],
            $this->_db->quote($this->_wishlist),
            (int)$itemId,
            $this->_convertToDriver($this->_db->quote($name)),
            $this->_convertToDriver($this->_db->quote($desc)),
            (int)$quantity,
            $this->_convertToDriver($this->_db->quote($price)),
            $this->_convertToDriver($this->_db->quote($prodid)),
            $this->_convertToDriver($this->_db->quote($prodid_type)),
            $this->_convertToDriver($this->_db->quote($vendors)),
            $this->_convertToDriver($this->_db->quote($category)),
            (int)$purchased,
            $this->_db->quote($purchased_on),
            $this->_convertToDriver($this->_db->quote($purchased_by)));

        /* Log the query at a DEBUG log level. */
        Horde::logMessage(sprintf('Genie_Driver_sql::add(): %s', $query),
                          __FILE__, __LINE__, PEAR_LOG_DEBUG);

        /* Attempt the insertion query. */
        $result = $this->_db->query($query);

        /* Return an error immediately if the query failed. */
        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        /* Log the creation of this item in the history log. */
        $history = &Horde_History::singleton();
        $history->log($this->getGUID($itemId), array('action' => 'add'), true);

        return $itemId;
    }

    /**
     * Modify an existing item.
     */
    function modify($itemId, $name, $desc, $quantity, $price, $prodid,
                    $prodid_type, $vendors, $category, $purchased,
                    $purchased_on = null, $purchased_by = '')
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        $query  = sprintf('UPDATE %s SET ', $this->_params['table']);
        $query .= sprintf('wish_name = %s, ',
                          $this->_convertToDriver($this->_db->quote($name)));
        $query .= sprintf('wish_desc = %s, ',
                          $this->_convertToDriver($this->_db->quote($desc)));
        $query .= sprintf('wish_quantity = %d, ', $quantity);
        $query .= sprintf('wish_price = %s, ',
                          $this->_convertToDriver($this->_db->quote($price)));
        $query .= sprintf('wish_prodid = %s, ',
                          $this->_convertToDriver($this->_db->quote($prodid)));
        $query .= sprintf('wish_prodid_type = %s, ',
                          $this->_convertToDriver($this->_db->quote($prodid_type)));
        $query .= sprintf('wish_vendors = %s, ',
                          $this->_convertToDriver($this->_db->quote($vendors)));
        $query .= sprintf('wish_category = %s, ', $this->_db->quote($category));
        $query .= sprintf('wish_purchased = %d, ', $purchased);
        $query .= sprintf('wish_purchased_on = %d, ', $purchased_on);
        $query .= sprintf('wish_purchased_by = %s ',
                          $this->_convertToDriver($this->_db->quote($purchased_by)));
        $query .= sprintf('WHERE wish_owner = %s AND wish_id = %d',
                          $this->_db->quote($this->_wishlist), $itemId);

        /* Log the query at a DEBUG log level. */
        Horde::logMessage(sprintf('Genie_Driver_sql::modify(): %s', $query),
                          __FILE__, __LINE__, PEAR_LOG_DEBUG);

        /* Attempt the update query. */
        $result = $this->_db->query($query);

        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        /* Log the modification of this item in the history log. */
        $history = &Horde_History::singleton();
        $history->log($this->getGUID($itemId), array('action' => 'modify'), true);

        return true;
    }

    function deleteItem($itemId)
    {
        $this->_connect();

        $query = sprintf('DELETE FROM %s WHERE wish_owner = %s AND wish_id = %s',
                         $this->_params['table'],
                         $this->_db->quote($this->_wishlist),
                         (int)$itemId);

        /* Log the query at a DEBUG log level. */
        Horde::logMessage(sprintf('Genie_Driver_sql::deleteItem(): %s', $query),
                          __FILE__, __LINE__, PEAR_LOG_DEBUG);

        /* Attempt the delete query. */
        $result = $this->_db->query($query);

        if (is_a($result, 'PEAR_Error')) {
            Horde::logMessage($result, __FILE__, __LINE__, PEAR_LOG_ERR);
            return $result;
        }

        /* Log the deletion of this item in the history log. */
        $history = &Horde_History::singleton();
        $history->log($this->getGUID($itemId), array('action' => 'delete'), true);

        return true;
    }

    function deleteAll()
    {
        $this->_connect();

        $query = sprintf('DELETE FROM %s WHERE wish_owner = %s',
                         $this->_params['table'],
                         $this->_db->quote($this->_wishlist));

        /* Log the query at a DEBUG log level. */
        Horde::logMessage(sprintf('Genie_Driver_sql::deleteAll(): %s', $query),
                          __FILE__, __LINE__, PEAR_LOG_DEBUG);

        /* Attempt the delete query. */
        $result = $this->_db->query($query);

        return is_a($result, 'PEAR_Error') ? $result : true;
    }

    /**
     * Retrieves the wishlist's items from the database.
     *
     * @return mixed  True on success, PEAR_Error on failure.
     */
    function retrieve()
    {
        $this->_connect();

        /* Build the SQL query. */
        $query = sprintf('SELECT * FROM %s WHERE wish_owner = %s',
                         $this->_params['table'], $this->_db->quote($this->_wishlist));

        /* Log the query at a DEBUG log level. */
        Horde::logMessage(sprintf('Genie_Driver_sql::retrieve(): %s', $query),
                          __FILE__, __LINE__, PEAR_LOG_DEBUG);

        /* Execute the query. */
        $result = $this->_db->query($query);

        if (!is_a($result, 'PEAR_Error')) {
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            if (is_a($row, 'PEAR_Error')) {
                return $row;
            }

            /* Store the retrieved values in a fresh $items list. */
            $this->_items = array();
            while ($row && !is_a($row, 'PEAR_Error')) {
                /* Create a new item based on this row's values. */
                $item = array();
                $item['item_id'] = $row['wish_id'];
                $item['wishlist_id'] = $this->_wishlist;
                $item['name'] = $this->_convertFromDriver($row['wish_name']);
                $item['desc'] = $this->_convertFromDriver($row['wish_desc']);
                $item['quantity'] = $row['wish_quantity'];
                $item['price'] = $this->_convertFromDriver($row['wish_price']);
                $item['prodid'] = $this->_convertFromDriver($row['wish_prodid']);
                $item['prodid_type'] = $this->_convertFromDriver($row['wish_prodid_type']);
                $item['vendors'] = $this->_convertFromDriver($row['wish_vendors']);
                $item['category'] = $this->_convertFromDriver($row['wish_category']);
                $item['purchased'] = $row['wish_purchased'];
                $item['purchased_on'] = $row['wish_purchased_on'];
                $item['purchased_by'] = $this->_convertFromDriver($row['wish_purchased_by']);
                $item['flags'] = 0;

                /* Add this new item to the $items list. */
                $this->_items[$row['wish_id']] = $item;

                /* Advance to the new row in the result set. */
                $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            }
            $result->free();
        } else {
            return $result;
        }

        return true;
    }

    /**
     * Attempts to open a connection to the SQL server.
     *
     * @return boolean    True on success; exits (Horde::fatal()) on error.
     */
    function _connect()
    {
        if (!$this->_connected) {
            Horde::assertDriverConfig($this->_params, 'storage',
                array('phptype', 'hostspec', 'username', 'database', 'charset'));

            if (!isset($this->_params['table'])) {
                $this->_params['table'] = 'genie_wishes';
            }

            /* Connect to the SQL server using the supplied parameters. */
            require_once 'DB.php';
            $this->_db = &DB::connect($this->_params,
                                      array('persistent' => !empty($this->_params['persistent'])));
            if (is_a($this->_db, 'PEAR_Error')) {
                Horde::fatal($this->_db, __FILE__, __LINE__);
            }

            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);

            $this->_connected = true;
        }

        return true;
    }

    /**
     * Converts a value from the driver's charset to the default charset.
     *
     * @param mixed $value  A value to convert.
     * @return mixed        The converted value.
     */
    function _convertFromDriver($value)
    {
        return String::convertCharset($value, $this->_params['charset']);
    }

    /**
     * Converts a value from the default charset to the driver's charset.
     *
     * @param mixed $value  A value to convert.
     * @return mixed        The converted value.
     */
    function _convertToDriver($value)
    {
        return String::convertCharset($value, NLS::getCharset(), $this->_params['charset']);
    }
}
